from django.db import models
from datetime import datetime
import re
import logging
import traceback
from django.contrib.auth.hashers import make_password, check_password
from django.core.validators import validate_email


RE_USERNAME = re.compile(r'^[a-zA-Z][a-zA-Z0-9_]{4,15}$')
RE_PASSWORD = re.compile(r'^(?=.*\d)(?=.*[a-zA-Z]).{8,16}$')


# Create your models here.
class User(models.Model):
    """
    这是用户数据库表对应的模型类
    Django支持ORM（对象关系映射）
    通俗来说就是无需使用sql语句操作数据库，仅仅需要使用models.Model父类提供的方法就可以实现你使用sql能办到的事情
    """
    username = models.CharField(verbose_name="用户名", unique=True, max_length=20)
    email = models.EmailField(verbose_name="邮箱", unique=True, validators=[validate_email, ])
    password = models.CharField("用户密码", max_length=1024)
    aboutMe = models.CharField(verbose_name="个人简介", max_length=1024, default="最美的我～")
    # auto_now_add 代表实例化对象时默认添加的时间
    createTime = models.DateTimeField(verbose_name="注册时间", auto_now_add=True)
    # auto_now 代表修改对象属性时默认保存当前时间
    lastActive = models.DateTimeField(verbose_name="最后活跃时间", auto_now=True)
    isDeleted = models.BooleanField(verbose_name="删除", default=False)

    def __str__(self):
        """
        打印实例化出来的对象时默认展示用户名
        :return:
        """
        return self.username

    @classmethod
    def make_password(cls, password):
        """
        使用Django的默认生成密码密文
        :param password:
        :return:
        """
        return make_password(password)

    @classmethod
    def check_password(cls, password):
        """
        使用Django的默认密码校验
        :param password:
        :return:
        """
        return check_password(password, cls.password)

    def save(self, *args, **kwargs):
        """
        重写保存方法，加入写入前数据验证
        :param args:
        :param kwargs:
        :return:
        """
        self.valid()
        super().save(*args, **kwargs)

    def valid(self):
        """
        验证数据是否合法
        :return:
        """
        try:
            for func in (self.username_valid,
                         self.password_valid,
                         self.email_valid):
                func()
        except Exception as err:
            logging.error(str(err))
            logging.error(traceback.format_exc())
            raise ValueError("数据验证失败！")

    def username_valid(self):
        """
        用户名验证
        :return:
        """
        if not re.match(RE_USERNAME, self.username):
            raise ValueError("用户名不合法！")

    def password_valid(self):
        """
        密码验证
        :return:
        """
        if not re.match(RE_PASSWORD, self.password):
            raise ValueError("密码不合法！")

    def email_valid(self):
        """
        邮箱验证
        :return:
        """
        validate_email(self.email)


class Label(models.Model):
    """
    标签
    """
    label = models.TextField(verbose_name="标签", name="label", max_length=64)
    createTime = models.DateTimeField(verbose_name="创建时间", auto_now_add=True)

    def __str__(self):
        """
        标签
        :return:
        """
        return self.label


class Article(models.Model):
    """
    文章数据库表模型
    """
    title = models.CharField(verbose_name="标题", max_length=256)
    content = models.TextField(verbose_name="内容", max_length=65536, default=None)
    createTime = models.DateTimeField(verbose_name="创建时间", auto_now_add=True)
    modifyTime = models.DateTimeField(verbose_name="最近修改时间", auto_now=True)
    # 外联标签表
    label = models.ForeignKey(Label, verbose_name="分类", on_delete=models.CASCADE)
    # 外键关联用户表
    author = models.ForeignKey(User, verbose_name="作者", on_delete=models.CASCADE)
    isDeleted = models.BooleanField(verbose_name="删除", default=False)
    isPrimary = models.BooleanField(verbose_name="仅个人可见", default=False)

    def __str__(self):
        """
        打印实例化出来的对象时默认展示文章标题
        :return:
        """
        return self.title


class Comment(models.Model):
    """
    文章评论数据库表模型
    """
    commentator = models.ForeignKey(User, verbose_name="评论者", on_delete=models.CASCADE)
    article = models.ForeignKey(Article, verbose_name="文章", on_delete=models.CASCADE)
    content = models.TextField(verbose_name="评论内容", max_length=65536, default=None)
    createTime = models.DateTimeField(verbose_name="评论时间", auto_now_add=True)
    isDeleted = models.BooleanField(verbose_name="删除", default=False)
    isPrimary = models.BooleanField(verbose_name="是否可见", default=False)


class Token(models.Model):
    """
    用户免登陆验证
    """
    token = models.TextField(verbose_name="token", name="token", max_length=64)
    createTime = models.DateTimeField(verbose_name="授权时间", auto_now_add=True)
    user = models.ForeignKey(User, verbose_name="授权用户", on_delete=models.CASCADE)

    def __str__(self):
        """
        打印实例化出来的对象时默认展示文章标题
        :return:
        """
        return self.token

